/* Algoritmul lui Lee
 * - Algoritm de Programare Dinamica
 * 
 * Problema Labirintului
 * Intr-un labirint de dimensiuni n x m, celulele libere
 * sunt marcate cu '_', iar celulele ocupate cu '#'.
 * Un soricel porneste de la poz (ip, jp) si trebuie sa ajunga
 * la branza, adica in celula (is, js).
 * 
 * Sa se determine numarul minim de pasi, pe care soricelul trebuie sa-i faca
 * de la (ip, jp) la (is, js).
 * 
 * Deplasarea se face din celula curenta, intr-un dintre celule adiacente.
 * 
 * 
 * 
 */ 

#include <fstream>
#include <algorithm>
#include <iomanip>
#include <vector>
#include <queue>
using namespace std;

ifstream fin("labirint.in");
ofstream fout("labirint.out");

// type Alias-uri

using PI  = pair<int, int>;
using VI  = vector<int>;
using VVI = vector<VI>;
using VC  = vector<char>;
using VVC = vector<VC>;


const int Dim = 101, Inf = 1e9; 
const int di[] = {-1, 0, 1, 0};   // variatiile de pozitii la deplasari spre N, E, S, V
const int dj[] = {0, 1, 0, -1}; 

VVI c;  //  c[i][j] - distanta minima de la (ip, jp) la 
                    // celula (i, j)
VVC L;
int n, m;
int ip, jp, is, js; // poz de plecare, respectiv sosire

void CitesteDate();
void Lee();
void Debug();
bool Ok(int i, int j);

int main()
{
	CitesteDate();
	Lee();
	Debug();
	fout << c[is][js];
}

void Lee()
{
	c = VVI(n + 1, VI(m + 1, Inf)); // se creaza matricea c[][]
	
	queue<PI> Q;
	c[ip][jp] = 0;
	Q.push({ip, jp});
	
	int iv, jv;
	int i, j;  // linia si coloana celulei aflate in capul cozii
	while (!Q.empty())    // cat timp coada nu e vida
	{
		tie(i, j) = Q.front(); // aflam celula din capul cozii
		Q.pop();               // si o stergem din coada
		
		for (int  d = 0; d < 4; ++d)
		{
			iv = i + di[d];
			jv = j + dj[d];
			
			if (Ok(iv, jv) && c[iv][jv] > c[i][j] + 1)
			{
				c[iv][jv] = c[i][j] + 1;
				Q.push({iv, jv});
			}
		}
	}
}

bool Ok(int i, int j)
{
	if (i < 1 || i > n || j < 1 || j > m) // in afara matricei
		return false;
	if (L[i][j] == '#') // obstacol
		return false;
		
	return true;
}


void Debug()
{
	for (int i = 1; i <= n; ++i)
	{
		for (int j = 1; j <= m; ++j)
			if (L[i][j] == '#')
				fout << " []";
			else
				if (c[i][j] != Inf)
					fout << setw(3) << c[i][j];
				else
					fout << " oo";
		fout << '\n';
	}
	
	fout << '\n';
}

void CitesteDate()
{
	fin >> n >> m;
	
	L = VVC(n + 1, VC(m + 1));
	fin >> ip >> jp >> is >> js;
	
	for (int i = 1; i <= n; ++i)
		for (int j = 1; j <= m; ++j)
			fin >> L[i][j];
}



